
% This function matches keypoints.  A match is accepted
%   only if its distance is less than distRatio times the distance to the
%   second closest match.

function [m,C] = fpmatch(fp1, fp2, umbral)

% fp1:  Keypoints imagen 1
% fp2:  Keypoints imagen 2
% umbral: Umbral para la correspondencia (distRatio) (opcional)

% Varlores devueltos
% m: FeatureMatch vector
% C: correspondencias (indices)

m = FeatureMatch.empty;
C = [];

if nargin<2
    return
end
if(size(fp1, 2) < 1 || size(fp2, 2) < 1 )
    return
end
if nargin<3
    umbral = 1.0;
end

% Extrae descriptores y coordenadas
des1 = zeros(size(fp1, 2), size(fp1(1).descriptor,1));
loc1 = zeros(size(fp1, 2), 2);
for i = 1:size(fp1, 2)
    des1(i,:) = fp1(i).descriptor';
    loc1(i,:) = [fp1(i).u_, fp1(i).v_];
end

des2 = zeros(size(fp2, 2), size(fp2(1).descriptor,1));
loc2 = zeros(size(fp2, 2), 2);
for i = 1:size(fp2, 2)
    des2(i,:) = fp2(i).descriptor';
    loc2(i,:) = [fp2(i).u_, fp2(i).v_];  
end


% For efficiency in Matlab, it is cheaper to compute dot products between
%  unit vectors rather than Euclidean distances.  Note that the ratio of 
%  angles (acos of dot products of unit vectors) is a close approximation
%  to the ratio of Euclidean distances for small angles.
%
% distRatio: Only keep matches in which the ratio of vector angles from the
%   nearest to second nearest neighbor is less than distRatio.
distRatio = umbral;   
distance = zeros(size(des1,1), 1);
match = zeros(size(des1,1), 1);

% normalise descriptors
des1 = des1 ./ repmat(sqrt(sum(des1.*des1,2)), [1 size(des1,2)]);
des2 = des2 ./ repmat(sqrt(sum(des2.*des2,2)), [1 size(des2,2)]);       

% For each descriptor in the first image, select its match to second image.
des2t = des2';                          % Precompute matrix transpose
for i = 1 : size(des1,1)
   dotprods = des1(i,:) * des2t;        % Computes vector of dot products
   [vals,indx] = sort(acos(dotprods));  % Take inverse cosine and sort results
   
   distance(i) = vals(1);   % save minimun cosine distance 
   % Check if nearest neighbor has angle less than distRatio times 2nd.

   if size(vals,2)==1 || distRatio >= 1.0 || (vals(1) < distRatio * vals(2))
      match(i) = indx(1);      
   else
      match(i) = 0;
   end
end

%GENERA LA MATRIZ DE RESULTADOS
%Por filas los puntos casados
%Por columnas (x,y) primera imagen, (x,y) segunda imagen

%Numero de puntos casados
num = sum(match > 0);
if num == 0
    return
end

C = zeros(2, num);
indice=1;

%Bucle para cada caracteristica encontrada
for i = 1: size(des1,1)
    %Caracteristica casada
    if (match(i) > 0)
        m(indice) = FeatureMatch(fp1(i), fp2(match(i)), distance(i));
        C(:,indice) = [ i ; match(i)];
        indice=indice+1;
    end
end

    
end







