
% This function matches keypoints.  A match is accepted
%   only if its distance is less than distRatio times the distance to the
%   second closest match.
%   If a figure is passed represents Keypoints matches


function [puntosMatch] = match(loc1, des1, loc2, des2, umbral, fig, ancho)

% Keypoints imagen 1
% loc1: matriz (coordenadas (x1,y1) (un Keypoint por fila)
% des1: matriz descriptores, (un descriptor por fila) 
% Keypoints imagen 2
% loc2: matriz (coordenadas (x2,y2) (un Keypoint por fila)
% des2: matriz descriptores, (un descriptor por fila) 
% umbral: Opcional Umbral para la correspondencia (distRatio)
% fig: Opcional: dentificador figura -> muestra las correspondecias
% ancho: Opcional ancho de imagen, para visualizar las correspondencias
%
% Varlor devuelto
% puntosMatch: Puntos casados. En cada fila x1,y1,x2,y2,dist

puntosMatch = [];
if nargin<4
    return
end

if size(loc1, 1) < 1 || size(loc1, 1) ~=size(des1, 1) || size(loc2, 1) < 1 || size(loc2, 1) ~=size(des2, 1)
    return
end

if nargin<5
    umbral = 1.0;
end

% For efficiency in Matlab, it is cheaper to compute dot products between
%  unit vectors rather than Euclidean distances.  Note that the ratio of 
%  angles (acos of dot products of unit vectors) is a close approximation
%  to the ratio of Euclidean distances for small angles.
%
% distRatio: Only keep matches in which the ratio of vector angles from the
%   nearest to second nearest neighbor is less than distRatio.
distRatio = umbral;   
distance = zeros(size(des1,1), 1);
match = zeros(size(des1,1), 1);

% normalise descriptors
des1 = des1 ./ repmat(sqrt(sum(des1.*des1,2)), [1 size(des1,2)]);
des2 = des2 ./ repmat(sqrt(sum(des2.*des2,2)), [1 size(des2,2)]);  

% For each descriptor in the first image, select its match to second image.
des2t = des2';                          % Precompute matrix transpose
for i = 1 : size(des1,1)
   dotprods = des1(i,:) * des2t;        % Computes vector of dot products
   [vals,indx] = sort(acos(dotprods));  % Take inverse cosine and sort results
    
   distance(i) = vals(1);   % save minimun cosine distance 
   % Check if nearest neighbor has angle less than distRatio times 2nd.
   if size(vals,2)==1 || (vals(1) < distRatio * vals(2))
      match(i) = indx(1);
   else
      match(i) = 0;
   end
end


%GENERA LA MATRIZ DE RESULTADOS
%Por filas los puntos casados
%Por columnas (x,y) primera imagen, (x,y) segunda imagen

%Numero de puntos casados
num = sum(match > 0);
puntosMatch=zeros(num,4);
indice=1;

%Bucle para cada caracteristica encontrada
for i = 1: size(des1,1)
    %Caracteristica casada
    if (match(i) > 0)
        puntosMatch(indice,1)=loc1(i,1);
        puntosMatch(indice,2)=loc1(i,2);
        puntosMatch(indice,3)=loc2(match(i),1);
        puntosMatch(indice,4)=loc2(match(i),2);
        puntosMatch(indice,5)=distance(i,1);
        indice=indice+1;
    end
end

    
%REPRESENTA LA IMAGEN
%Ambas imagenes con correspondencias
if (nargin>6)
    figure(fig);
   
    hold on;
    for i = 1: size(des1,1)
        if (match(i) > 0)
            line([loc1(i,1) loc2(match(i),1)+ancho], ...
                [loc1(i,2) loc2(match(i),2)], 'Color', 'c');
        end
    end
    hold off;
end



end







